@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -22,28 +22,70 @@
 
 -module(ssl_session_cache_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
-
 -include_lib("common_test/include/ct.hrl").
 
--define(DELAY, 500).
--define(SLEEP, 1000).
--define(TIMEOUT, 60000).
--define(LONG_TIMEOUT, 600000).
--define(MAX_TABLE_SIZE, 5).
+%% Callback functions
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2,
+         init_per_testcase/2,
+         end_per_testcase/2]).
+
+%% Testcases
+-export([session_cleanup/0,
+         session_cleanup/1,
+         session_cache_process_list/0,
+         session_cache_process_list/1,
+         session_cache_process_mnesia/0,
+         session_cache_process_mnesia/1,
+         client_unique_session/0,
+         client_unique_session/1,
+         max_table_size/0,
+         max_table_size/1,
+         save_specific_session/0,
+         save_specific_session/1
+        ]).
+
+%% Apply export
+-export([connection_info_result/1]).
 
 -behaviour(ssl_session_cache_api).
 
 %% For the session cache tests
 -export([init/1, terminate/1, lookup/2, update/3,
-	 delete/2, foldl/3, select_session/2]).
+         size/1, delete/2, foldl/3, select_session/2]).
+
+-define(SLEEP, 1000).
+-define(TIMEOUT, {seconds, 20}).
+-define(MAX_TABLE_SIZE, 5).
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
 
-all() ->
+
+all() -> 
+    [
+     {group, 'tlsv1.2'},
+     {group, 'tlsv1.1'},
+     {group, 'tlsv1'},
+     {group, 'dtlsv1.2'},
+     {group, 'dtlsv1'}
+    ].
+
+groups() ->
+    [{'dtlsv1.2', [], session_tests()},
+     {'dtlsv1', [], session_tests()},
+     {'tlsv1.2', [], session_tests()},
+     {'tlsv1.1', [], session_tests()},
+     {'tlsv1', [], session_tests()}
+    ].
+
+
+session_tests() ->
     [session_cleanup,
      session_cache_process_list,
      session_cache_process_mnesia,
@@ -52,9 +94,6 @@ all() ->
      save_specific_session
      ].
 
-groups() ->
-    [].
-
 init_per_suite(Config0) ->
     catch crypto:stop(),
     try crypto:start() of
@@ -70,11 +109,11 @@ end_per_suite(_Config) ->
     ssl:stop(),
     application:stop(crypto).
 
-init_per_group(_GroupName, Config) ->
-    Config.
+init_per_group(GroupName, Config) ->
+    ssl_test_lib:init_per_group(GroupName, Config). 
 
-end_per_group(_GroupName, Config) ->
-    Config.
+end_per_group(GroupName, Config) ->
+  ssl_test_lib:end_per_group(GroupName, Config).
 
 init_per_testcase(session_cache_process_list, Config) ->
     init_customized_session_cache(list, Config);
@@ -84,42 +123,52 @@ init_per_testcase(session_cache_process_mnesia, Config) ->
     init_customized_session_cache(mnesia, Config);
 
 init_per_testcase(session_cleanup, Config) ->
+    Versions = ssl_test_lib:protocol_version(Config),
     ssl:stop(),
     application:load(ssl),
+    ssl_test_lib:set_protocol_versions(Versions),
     application:set_env(ssl, session_lifetime, 5),
-    application:set_env(ssl, session_delay_cleanup_time, ?DELAY),
     ssl:start(),
-    ct:timetrap({seconds, 20}),
+    ssl_test_lib:ct_log_supported_protocol_versions(Config),
+    ct:timetrap(?TIMEOUT),
     Config;
 
 init_per_testcase(client_unique_session, Config) ->
-    ct:timetrap({seconds, 40}),
+    ct:timetrap(?TIMEOUT),
+    ssl_test_lib:ct_log_supported_protocol_versions(Config),
     Config;
 init_per_testcase(save_specific_session, Config) ->
+    Versions = ssl_test_lib:protocol_version(Config),
     ssl_test_lib:clean_start(),
-    ct:timetrap({seconds, 5}),
+    ssl_test_lib:set_protocol_versions(Versions),
+    ct:timetrap(?TIMEOUT),
     Config;
 init_per_testcase(max_table_size, Config) ->
+    Versions = ssl_test_lib:protocol_version(Config),
     ssl:stop(),
     application:load(ssl),
+    ssl_test_lib:set_protocol_versions(Versions),
     application:set_env(ssl, session_cache_server_max, ?MAX_TABLE_SIZE),
     application:set_env(ssl, session_cache_client_max, ?MAX_TABLE_SIZE),
-    application:set_env(ssl, session_delay_cleanup_time, ?DELAY),	
-    ssl:start(),			  
+    ssl:start(),	
+    ssl_test_lib:ct_log_supported_protocol_versions(Config),
     ct:timetrap({seconds, 40}),
     Config.
 
 init_customized_session_cache(Type, Config) ->
+    Versions = ssl_test_lib:protocol_version(Config),
     ssl:stop(),
     application:load(ssl),
+    ssl_test_lib:set_protocol_versions(Versions),
     application:set_env(ssl, session_cb, ?MODULE),
     application:set_env(ssl, session_cb_init_args, [{type, Type}]),
     ssl:start(),
+    ssl_test_lib:ct_log_supported_protocol_versions(Config),
     catch (end_per_testcase(list_to_atom("session_cache_process" ++ atom_to_list(Type)),
 	   Config)),
     ets:new(ssl_test, [named_table, public, set]),
     ets:insert(ssl_test, {type, Type}),
-    ct:timetrap({seconds, 20}),
+    ct:timetrap(?TIMEOUT),
     Config.
 
 end_per_testcase(session_cache_process_list, Config) ->
@@ -133,7 +182,6 @@ end_per_testcase(session_cache_process_mnesia, Config) ->
     ssl:start(),
     end_per_testcase(default_action, Config);
 end_per_testcase(session_cleanup, Config) ->
-    application:unset_env(ssl, session_delay_cleanup_time),
     application:unset_env(ssl, session_lifetime),
     end_per_testcase(default_action, Config);
 end_per_testcase(max_table_size, Config) ->
@@ -155,8 +203,8 @@ client_unique_session() ->
       "sets up many connections"}].
 client_unique_session(Config) when is_list(Config) ->
     process_flag(trap_exit, true),
-    ClientOpts = proplists:get_value(client_rsa_verify_opts, Config),
-    ServerOpts = proplists:get_value(server_rsa_opts, Config),
+    ClientOpts = ssl_test_lib:ssl_options(client_rsa_verify_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_rsa_opts, Config),
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
     Server =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
@@ -199,43 +247,30 @@ session_cleanup(Config) when is_list(Config) ->
 	ssl_test_lib:start_client([{node, ClientNode},
 				   {port, Port}, {host, Hostname},
 				   {mfa, {ssl_test_lib, no_result, []}},
-				   {from, self()},  {options, ClientOpts}]),
+				   {from, self()},  {options, [{reuse_sessions, save} | ClientOpts]}]),
     SessionInfo =
 	receive
 	    {Server, Info} ->
 		Info
 	end,
 
-    %% Make sure session is registered
-    ct:sleep(?SLEEP*2),
-
     {status, _, _, StatusInfo} = sys:get_status(whereis(ssl_manager)),
     [_, _,_, _, Prop] = StatusInfo,
     State = ssl_test_lib:state(Prop),
     ClientCache = element(2, State),
-    ServerCache = element(3, State),
-    SessionTimer = element(7, State),
+    SessionTimer = element(6, State),
 
     Id = proplists:get_value(session_id, SessionInfo),
     CSession = ssl_session_cache:lookup(ClientCache, {{Hostname, Port}, Id}),
-    SSession = ssl_session_cache:lookup(ServerCache, {Port, Id}),
 
     true = CSession =/= undefined,
-    true = SSession =/= undefined,
 
     %% Make sure session has expired and been cleaned up
     check_timer(SessionTimer),
-    ct:sleep(?DELAY *2),  %% Delay time + some extra time
-
-    {ServerDelayTimer, ClientDelayTimer} = get_delay_timers(),
-
-    check_timer(ServerDelayTimer),
-    check_timer(ClientDelayTimer),
-
+    
     ct:sleep(?SLEEP),  %% Make sure clean has had time to run
-
+    
     undefined = ssl_session_cache:lookup(ClientCache, {{Hostname, Port}, Id}),
-    undefined = ssl_session_cache:lookup(ServerCache, {Port, Id}),
 
     process_flag(trap_exit, false),
     ssl_test_lib:close(Server),
@@ -259,8 +294,8 @@ save_specific_session() ->
      }].
 save_specific_session(Config) when is_list(Config) ->
     process_flag(trap_exit, true),
-    ClientOpts = proplists:get_value(client_rsa_verify_opts, Config),
-    ServerOpts = proplists:get_value(server_rsa_opts, Config),
+    ClientOpts = ssl_test_lib:ssl_options(client_rsa_verify_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_rsa_opts, Config),
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
     Server =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
@@ -342,13 +377,10 @@ max_table_size(Config) when is_list(Config) ->
     [_, _,_, _, Prop] = StatusInfo,
     State = ssl_test_lib:state(Prop),
     ClientCache = element(2, State),	
-    ServerCache = element(3, State),
-    N = ssl_session_cache:size(ServerCache),
     M = ssl_session_cache:size(ClientCache),
-    ct:pal("~p",[{N, M}]),			
+    ct:pal("~p",[M]),
     ssl_test_lib:close(Server, 500),
     ssl_test_lib:close(LastClient),
-    true = N =< ?MAX_TABLE_SIZE,
     true = M =< ?MAX_TABLE_SIZE.
 
 %%--------------------------------------------------------------------
@@ -362,7 +394,7 @@ init(Opts) ->
 	mnesia ->
 	    mnesia:start(),
 	    Name = atom_to_list(proplists:get_value(role, Opts)),
-	    TabName = list_to_atom(Name ++ "sess_cache"),
+	    TabName = list_to_atom(Name ++ "sess_cache" ++ erlang:pid_to_list(self())),
 	    {atomic,ok} = mnesia:create_table(TabName, []),
 	    TabName
     end.
@@ -380,6 +412,16 @@ terminate(Cache) ->
 		mnesia:delete_table(Cache)
     end.
 
+size(Cache) ->
+    case session_cb() of
+	list ->
+            Cache ! {self(), size},
+            receive {Cache, Res} -> Res end;
+        mnesia ->
+            mnesia:table_info(Cache, size)
+    end.
+
+
 lookup(Cache, Key) ->
     case session_cb() of
 	list ->
@@ -416,7 +458,7 @@ delete(Cache, Key) ->
 	mnesia ->
 	    {atomic, ok} =
 		mnesia:transaction(fun() ->
-					   mnesia:delete(Cache, Key)
+					   mnesia:delete(Cache, Key, write)
 				   end)
     end.
 
@@ -429,22 +471,26 @@ foldl(Fun, Acc, Cache) ->
 	    Foldl = fun() ->
 			    mnesia:foldl(Fun, Acc, Cache)
 		    end,
-	    {atomic, Res} = mnesia:transaction(Foldl),
-	    Res
+	    case mnesia:transaction(Foldl) of
+                {atomic, {_,Key, Value}} ->
+                    {Key, Value};
+                Error ->
+                    Error
+            end
     end.
 
 select_session(Cache, PartialKey) ->
     case session_cb() of
 	list ->
-	    Cache ! {self(),select_session, PartialKey},
+            Cache ! {self(),select_session, PartialKey},
 	    receive
-		{Cache, Res} ->
+		{_Cache, Res} ->
 		    Res
 	    end;
 	mnesia ->
 	    Sel = fun() ->
 			  mnesia:select(Cache,
-					[{{Cache,{PartialKey,'_'}, '$1'},
+					[{{Cache, {PartialKey,'_'}, '$1'},
 					  [],['$1']}])
 		  end,
 	    {atomic, Res} = mnesia:transaction(Sel),
@@ -455,6 +501,9 @@ session_loop(Sess) ->
     receive
 	terminate ->
 	    ok;
+        {Pid, size} ->
+            Pid ! {self(), length(Sess)},
+            session_loop(Sess);
 	{Pid, lookup, Key} ->
 	    case lists:keysearch(Key,1,Sess) of
 		{value, {Key,Value}} ->
@@ -481,7 +530,13 @@ session_loop(Sess) ->
 	    Sessions = lists:foldl(Sel, [], Sess),
 	    Pid ! {self(), Sessions},
 	    session_loop(Sess)
-    end.
+     end.
+%%--------------------------------------------------------------------
+%%% callback functions
+%%--------------------------------------------------------------------
+
+connection_info_result(Socket) ->
+    ssl:connection_information(Socket, [protocol, cipher_suite]).
 
 %%--------------------------------------------------------------------
 %%% Internal functions
@@ -510,38 +565,18 @@ clients_start(Server, ClientNode, Hostname, Port, ClientOpts, N) ->
     wait_for_server(),
     clients_start(Server, ClientNode, Hostname, Port, ClientOpts, N-1).
 	
-connection_info_result(Socket) ->
-    ssl:connection_information(Socket, [protocol, cipher_suite]).
 
 check_timer(Timer) ->
     case erlang:read_timer(Timer) of
 	false ->
 	    {status, _, _, _} = sys:get_status(whereis(ssl_manager)),
-	    timer:sleep(?SLEEP),
+	    ct:sleep(?SLEEP),
 	    {status, _, _, _} = sys:get_status(whereis(ssl_manager)),
 	    ok;
 	Int ->
 	    ct:sleep(Int),
 	    check_timer(Timer)
     end.
-
-get_delay_timers() ->
-    {status, _, _, StatusInfo} = sys:get_status(whereis(ssl_manager)),
-    [_, _,_, _, Prop] = StatusInfo,
-    State = ssl_test_lib:state(Prop),
-    case element(8, State) of
-	{undefined, undefined} ->
-	    ct:sleep(?SLEEP),
-	    get_delay_timers();
-	{undefined, _} ->
-	    ct:sleep(?SLEEP),
-	    get_delay_timers();
-	{_, undefined} ->
-	    ct:sleep(?SLEEP),
-	    get_delay_timers();
-	DelayTimers ->
-	    DelayTimers
-    end.
     
 wait_for_server() ->
     ct:sleep(100).	
