@@ -21,12 +21,44 @@
 
 -module(ssl_sni_SUITE).
 
--compile(export_all).
-
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 -include_lib("kernel/include/inet.hrl").
 
+%% Callback functions
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2,
+         init_per_testcase/2,
+         end_per_testcase/2]).
+
+%% Testcases
+-export([no_sni_header/1,
+         sni_match/1,
+         sni_no_match/1,
+         no_sni_header_fun/1,
+         sni_match_fun/1,
+         sni_no_match_fun/1,
+         dns_name/1,
+         ip_fallback/1,
+         no_ip_fallback/1,
+         dns_name_reuse/1,
+         customize_hostname_check/0,
+         customize_hostname_check/1,
+         sni_no_trailing_dot/0,
+         sni_no_trailing_dot/1,
+         hostname_trailing_dot/0,
+         hostname_trailing_dot/1
+        ]).
+
+%% Apply export
+-export([send_and_hostname/1,
+         recv_and_certificate/1
+        ]).
+
 -define(TIMEOUT, {seconds, 6}).
 
 %%--------------------------------------------------------------------
@@ -135,41 +167,46 @@ sni_no_match_fun(Config) ->
 dns_name(Config) ->
     Hostname = "OTP.test.server",
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions,  [#'Extension'{extnID = 
-                                                                                                                ?'id-ce-subjectAltName',
-                                                                                                            extnValue = [{dNSName, Hostname}],
-                                                                                                            critical = false}]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions,  [#'Extension'{extnID =
+                                                                                   ?'id-ce-subjectAltName',
+                                                                               extnValue = [{dNSName, Hostname}],
+                                                                               critical = false}]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     unsuccessfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], undefined, Config),
-    successfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, Hostname} | ClientConf], undefined, Config),
-    unsuccessfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, "foo"} | ClientConf], undefined, Config),
-    successfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, disable} | ClientConf], undefined, Config).
+    successfull_connect(ServerConf, [{verify, verify_peer},
+                                     {server_name_indication, Hostname} | ClientConf], undefined, Config),
+    unsuccessfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, "foo"} | ClientConf],
+                          undefined, Config),
+    successfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, disable} | ClientConf],
+                        undefined, Config).
 
 ip_fallback(Config) ->
     Hostname = net_adm:localhost(),
     {ok, #hostent{h_addr_list = [IP |_]}} = inet:gethostbyname(net_adm:localhost()),
     IPStr = tuple_to_list(IP),
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions, [#'Extension'{extnID = 
-                                                                                                               ?'id-ce-subjectAltName',
-                                                                                                           extnValue = [{dNSName, Hostname},
-                                                                                                                        {iPAddress, IPStr}],
-                                                                                                           critical = false}]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions, [#'Extension'{extnID =
+                                                                                  ?'id-ce-subjectAltName',
+                                                                              extnValue = [{dNSName, Hostname},
+                                                                                           {iPAddress, IPStr}],
+                                                                              critical = false}]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     successfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], Hostname, Config),
     successfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], IP, Config).
 
@@ -177,42 +214,44 @@ no_ip_fallback(Config) ->
     Hostname = net_adm:localhost(),
     {ok, #hostent{h_addr_list = [IP |_]}} = inet:gethostbyname(net_adm:localhost()),
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions, [#'Extension'{extnID = 
-                                                                                                               ?'id-ce-subjectAltName',
-                                                                                                           extnValue = [{dNSName, Hostname}],
-                                                                                                           critical = false}]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}
-                                                                               ]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions, [#'Extension'{extnID =
+                                                                                  ?'id-ce-subjectAltName',
+                                                                              extnValue = [{dNSName, Hostname}],
+                                                                              critical = false}]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}
+                                                  ]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     successfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], Hostname, Config),
     unsuccessfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], IP, Config).
 
 dns_name_reuse(Config) ->
     SNIHostname = "OTP.test.server",
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions,  [#'Extension'{extnID = 
-                                                                                                                ?'id-ce-subjectAltName',
-                                                                                                            extnValue = [{dNSName, SNIHostname}],
-                                                                                                            critical = false}
-                                                                                              ]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}
-                                                                               ]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions,  [#'Extension'{extnID =
+                                                                                   ?'id-ce-subjectAltName',
+                                                                               extnValue = [{dNSName, SNIHostname}],
+                                                                               critical = false}
+                                                                 ]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}
+                                                  ]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
-  
+
     unsuccessfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], undefined, Config),
     
     Server = 
@@ -351,27 +390,8 @@ hostname_trailing_dot(Config) when is_list(Config) ->
     end.
 
 %%--------------------------------------------------------------------
-%% Internal Functions ------------------------------------------------
+%% Callback Functions ------------------------------------------------
 %%--------------------------------------------------------------------
-ssl_recv(SSLSocket, Expect) ->
-    ssl_recv(SSLSocket, "", Expect).
-
-ssl_recv(SSLSocket, CurrentData, ExpectedData) ->
-    receive
-        {ssl, SSLSocket, Data} ->
-            NeweData = CurrentData ++ Data,
-            case NeweData of
-                ExpectedData ->
-                    ok;
-                _  ->
-                    ssl_recv(SSLSocket, NeweData, ExpectedData)
-            end;
-        Other ->
-            ct:fail({unexpected_message, Other})
-    after 4000 ->
-            ct:fail({timeout, CurrentData, ExpectedData})
-    end.
-
 send_and_hostname(SSLSocket) ->
     ssl:send(SSLSocket, "OK"),
     case  ssl:connection_information(SSLSocket, [sni_hostname]) of
@@ -381,18 +401,6 @@ send_and_hostname(SSLSocket) ->
 	    undefined
     end.
 
-rdnPart([[#'AttributeTypeAndValue'{type=Type, value=Value} | _] | _], Type) -> 
-    Value;
-rdnPart([_ | Tail], Type) -> 
-    rdnPart(Tail, Type);
-rdnPart([], _) -> 
-    unknown.
-
-rdn_to_string({utf8String, Binary}) ->
-    erlang:binary_to_list(Binary);
-rdn_to_string({printableString, String}) ->
-    String.
-
 recv_and_certificate(SSLSocket) ->
     ssl_recv(SSLSocket, "OK"),
     {ok, PeerCert} = ssl:peercert(SSLSocket),
@@ -401,6 +409,9 @@ recv_and_certificate(SSLSocket) ->
     ct:log("Subject of certificate received from server: ~p", [Subject]),
     rdn_to_string(rdnPart(Subject, ?'id-at-commonName')).
 
+%%--------------------------------------------------------------------
+%% Internal Functions ------------------------------------------------
+%%--------------------------------------------------------------------
 run_sni_fun_handshake(Config, SNIHostname, ExpectedSNIHostname, ExpectedCN) ->
     ct:log("Start running handshake for sni_fun, Config: ~p, SNIHostname: ~p, "
 	   "ExpectedSNIHostname: ~p, ExpectedCN: ~p", 
@@ -502,3 +513,33 @@ trailing_dot_hostname(HostName) ->
             {skip, "Trailing dot conf not possible"}
      end.
 
+ssl_recv(SSLSocket, Expect) ->
+    ssl_recv(SSLSocket, "", Expect).
+
+ssl_recv(SSLSocket, CurrentData, ExpectedData) ->
+    receive
+        {ssl, SSLSocket, Data} ->
+            NeweData = CurrentData ++ Data,
+            case NeweData of
+                ExpectedData ->
+                    ok;
+                _  ->
+                    ssl_recv(SSLSocket, NeweData, ExpectedData)
+            end;
+        Other ->
+            ct:fail({unexpected_message, Other})
+    after 4000 ->
+            ct:fail({timeout, CurrentData, ExpectedData})
+    end.
+
+rdnPart([[#'AttributeTypeAndValue'{type=Type, value=Value} | _] | _], Type) ->
+    Value;
+rdnPart([_ | Tail], Type) ->
+    rdnPart(Tail, Type);
+rdnPart([], _) ->
+    unknown.
+
+rdn_to_string({utf8String, Binary}) ->
+    erlang:binary_to_list(Binary);
+rdn_to_string({printableString, String}) ->
+    String.
