@@ -21,14 +21,75 @@
 %%
 -module(tls_api_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
 -include_lib("common_test/include/ct.hrl").
 -include_lib("ssl/src/ssl_record.hrl").
 -include_lib("ssl/src/ssl_internal.hrl").
 -include_lib("ssl/src/ssl_api.hrl").
 -include_lib("ssl/src/tls_handshake.hrl").
 
+%% Common test
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         init_per_group/2,
+         end_per_suite/1,
+         end_per_group/2
+        ]).
+
+%% Test cases
+-export([tls_upgrade/0,
+         tls_upgrade/1,
+         tls_upgrade_with_timeout/0,
+         tls_upgrade_with_timeout/1,
+         tls_downgrade/0,
+         tls_downgrade/1,
+         tls_shutdown/0,
+         tls_shutdown/1,
+         tls_shutdown_write/0,
+         tls_shutdown_write/1,
+         tls_shutdown_both/0,
+         tls_shutdown_both/1,
+         tls_shutdown_error/0,
+         tls_shutdown_error/1,
+         tls_client_closes_socket/0,
+         tls_client_closes_socket/1,
+         tls_closed_in_active_once/0,
+         tls_closed_in_active_once/1,
+         tls_tcp_msg/0,
+         tls_tcp_msg/1,
+         tls_tcp_msg_big/0,
+         tls_tcp_msg_big/1,
+         tls_dont_crash_on_handshake_garbage/0,
+         tls_dont_crash_on_handshake_garbage/1,
+         tls_tcp_error_propagation_in_active_mode/0,
+         tls_tcp_error_propagation_in_active_mode/1,
+         peername/0,
+         peername/1,
+         sockname/0,
+         sockname/1,
+         tls_server_handshake_timeout/0,
+         tls_server_handshake_timeout/1,
+         transport_close/0,
+         transport_close/1,
+         emulated_options/0,
+         emulated_options/1,
+         accept_pool/0,
+         accept_pool/1,
+         reuseaddr/0,
+         reuseaddr/1
+        ]).
+
+%% Apply export
+-export([upgrade_result/1,
+         tls_downgrade_result/2,
+         tls_shutdown_result/2,
+         tls_shutdown_write_result/2,
+         tls_shutdown_both_result/2,
+         tls_socket_options_result/5,
+         receive_msg/1
+        ]).
+
+-define(TIMEOUT, {seconds, 10}).
 -define(SLEEP, 500).
 
 %%--------------------------------------------------------------------
@@ -40,8 +101,7 @@ all() ->
      {group, 'tlsv1.3'},
      {group, 'tlsv1.2'},
      {group, 'tlsv1.1'},
-     {group, 'tlsv1'},
-     {group, 'sslv3'}
+     {group, 'tlsv1'}
     ].
 
 groups() ->
@@ -49,8 +109,7 @@ groups() ->
      {'tlsv1.3', [], api_tests() -- [sockname]},
      {'tlsv1.2', [],  api_tests()},
      {'tlsv1.1', [],  api_tests()},
-     {'tlsv1', [],  api_tests()},
-     {'sslv3', [],  api_tests() ++ [ssl3_cipher_suite_limitation]}
+     {'tlsv1', [],  api_tests()}
     ].
 
 api_tests() ->
@@ -92,33 +151,11 @@ end_per_suite(_Config) ->
     application:unload(ssl),
     application:stop(crypto).
 
-
 init_per_group(GroupName, Config) ->
-    case ssl_test_lib:is_tls_version(GroupName) of
-	true ->
-	    case ssl_test_lib:sufficient_crypto_support(GroupName) of
-		true ->
-		    ssl_test_lib:init_tls_version(GroupName, Config);
-		false ->
-		    {skip, "Missing crypto support"}
-	    end;
-	_ ->
-	    ssl:start(),
-	    Config
-    end.
+    ssl_test_lib:init_per_group(GroupName, Config). 
 
 end_per_group(GroupName, Config) ->
-    case ssl_test_lib:is_tls_version(GroupName) of
-        true ->
-            ssl_test_lib:clean_tls_version(Config);
-        false ->
-            Config
-    end.
-
-init_per_testcase(_TestCase, Config) ->
-    ssl_test_lib:ct_log_supported_protocol_versions(Config),
-    ct:timetrap({seconds, 10}),
-    Config.
+  ssl_test_lib:end_per_group(GroupName, Config).
 
 end_per_testcase(_TestCase, Config) ->     
     Config.
@@ -498,13 +535,13 @@ peername(Config) when is_list(Config) ->
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
     Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0}, 
 					{from, self()}, 
-			   {mfa, {?MODULE, peername_result, []}},
+			   {mfa, {ssl, peername, []}},
 			   {options, ServerOpts}]),
     Port = ssl_test_lib:inet_port(Server),
     Client = ssl_test_lib:start_client([{node, ClientNode}, {port, Port}, 
 					{host, Hostname},
 					{from, self()}, 
-					{mfa, {?MODULE, peername_result, []}},
+					{mfa, {ssl, peername, []}},
 					{options, [{port, 0} | ClientOpts]}]),
     
     ClientPort = ssl_test_lib:inet_port(Client),
@@ -530,13 +567,13 @@ sockname(Config) when is_list(Config) ->
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
     Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0}, 
 					{from, self()}, 
-			   {mfa, {?MODULE, sockname_result, []}},
+			   {mfa, {ssl, sockname, []}},
 			   {options, ServerOpts}]),
     Port = ssl_test_lib:inet_port(Server),
     Client = ssl_test_lib:start_client([{node, ClientNode}, {port, Port}, 
 					{host, Hostname},
                                         {from, self()}, 
-                                        {mfa, {?MODULE, sockname_result, []}},
+                                        {mfa, {ssl, sockname, []}},
                                         {options, [{port, 0} | ClientOpts]}]),
                                        
     ClientPort = ssl_test_lib:inet_port(Client),
@@ -605,39 +642,6 @@ transport_close(Config) when is_list(Config) ->
     {error, _} = ssl:send(SslS, "Hello world").
 
 %%--------------------------------------------------------------------
-ssl3_cipher_suite_limitation()  ->
-    [{doc,"Test a SSLv3 client cannot negotiate a TLSv* cipher suite."}].
-ssl3_cipher_suite_limitation(Config) when is_list(Config) ->
-    
-    {_ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
-    ServerOpts = ssl_test_lib:ssl_options(server_rsa_opts, Config),
-    
-    Server = ssl_test_lib:start_server_error([{node, ServerNode}, {port, 0},
-					      {from, self()},
-					      {options, ServerOpts}]),
-    Port = ssl_test_lib:inet_port(Server),
-    
-    {ok, Socket} = gen_tcp:connect(Hostname, Port, [binary, {active, false}]),
-    ok = gen_tcp:send(Socket, 
-		      <<22, 3,0, 49:16, % handshake, SSL 3.0, length
-			1, 45:24, % client_hello, length
-			3,0, % SSL 3.0
-			16#deadbeef:256, % 32 'random' bytes = 256 bits
-			0, % no session ID
-			%% three cipher suites -- null, one with sha256 hash and one with sha hash
-			6:16, 0,255, 0,61, 0,57, 
-			1, 0 % no compression
-		      >>),
-    {ok, <<22, RecMajor:8, RecMinor:8, _RecLen:16, 2, HelloLen:24>>} = gen_tcp:recv(Socket, 9, 10000),
-    {ok, <<HelloBin:HelloLen/binary>>} = gen_tcp:recv(Socket, HelloLen, 5000),
-    ServerHello = tls_handshake:decode_handshake({RecMajor, RecMinor}, 2, HelloBin),
-    case ServerHello of
-	#server_hello{server_version = {3,0}, cipher_suite = <<0,57>>} -> 
-	    ok;
-	_ ->
-	    ct:fail({unexpected_server_hello, ServerHello})
-    end.
-%%--------------------------------------------------------------------
 emulated_options() ->
     [{doc,"Test API function getopts/2 and setopts/2"}].
 
@@ -792,7 +796,8 @@ tls_downgrade_result(Socket, Pid) ->
 	    gen_tcp:send(TCPSocket, "Downgraded"),
             receive 
                 {tcp, TCPSocket, <<"Downgraded">>} ->
-	             ok;
+                    ct:sleep(?SLEEP),
+                    ok;
                 {tcp_closed, TCPSocket} ->
                     ct:fail("Did not receive TCP data"),
 	            ok;
@@ -859,12 +864,6 @@ receive_msg(_) ->
 	   Msg
     end.
  
-sockname_result(S) ->
-    ssl:sockname(S).
-
-peername_result(S) ->
-    ssl:peername(S).
-
 tls_socket_options_result(Socket, Options, DefaultValues, NewOptions, NewValues) ->
     %% Test get/set emulated opts
     {ok, DefaultValues} = ssl:getopts(Socket, Options), 
